#include <ros/ros.h>
#include <tf/transform_broadcaster.h>
#include <nav_msgs/Odometry.h>

std::string target_frame;

void poseCallback(const nav_msgs::OdometryConstPtr& msg){
  static tf::TransformBroadcaster br;
  tf::Transform transform;
  transform.setOrigin(tf::Vector3(msg->pose.pose.position.x, msg->pose.pose.position.y, msg->pose.pose.position.z));
  transform.setRotation(tf::Quaternion(msg->pose.pose.orientation.x,msg->pose.pose.orientation.y,msg->pose.pose.orientation.z,msg->pose.pose.orientation.w));
  br.sendTransform(tf::StampedTransform(transform, msg->header.stamp, "world", target_frame));
}

int main(int argc, char** argv){
  ros::init(argc, argv, "my_tf_broadcaster");
  if (argc != 3){
    ROS_ERROR("need Odometry message and target frame name as argument");
    return -1;
  }
  std::string p3d_message = argv[1];
  target_frame = argv[2];

  ros::NodeHandle node("~");
  ros::Subscriber sub = node.subscribe(p3d_message, 1, &poseCallback);

  ros::spin();
  return 0;
}

