/*********************************************************************
* Software License Agreement (BSD License)
*
*  Copyright (c) 2008, Willow Garage, Inc.
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*   * Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*   * Redistributions in binary form must reproduce the above
*     copyright notice, this list of conditions and the following
*     disclaimer in the documentation and/or other materials provided
*     with the distribution.
*   * Neither the name of the Willow Garage nor the names of its
*     contributors may be used to endorse or promote products derived
*     from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
*  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
*  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
*  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
*  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
*  POSSIBILITY OF SUCH DAMAGE.
*********************************************************************/

/* Author: Wim Meeussen */

#include <string>
#include <gtest/gtest.h>
#include <ros/ros.h>
#include <urdf/model.h>
#include <std_msgs/Bool.h>
#include <boost/thread.hpp>

namespace pr2_bringup_gazebo_demo{

int g_argc;
char** g_argv;

class TestCalibration : public testing::Test
{
public:

  /// constructor
  TestCalibration() 
  {
    calibrated_ = false;
    ros::NodeHandle nh;
    sub_ = nh.subscribe("calibrated", 1, &TestCalibration::calCb, this);
  }
  
  /// Destructor
  ~TestCalibration() {}

  // callback
  void calCb(const std_msgs::BoolConstPtr& msg)
  {
    calibrated_ = msg->data;
  }

  volatile bool calibrated_;

private:
  ros::Subscriber sub_;
};




TEST_F(TestCalibration, test)
{
  while (ros::ok() && !this->calibrated_)
    ros::Duration(0.1).sleep();
  ASSERT_TRUE(this->calibrated_);

  SUCCEED();
}


}

int main(int argc, char** argv)
{
  testing::InitGoogleTest(&argc, argv);
  ros::init(argc, argv, "test_pr2_calibration");

  boost::thread spinner(boost::bind(&ros::spin));
  int res = RUN_ALL_TESTS();
  spinner.interrupt();
  spinner.join();

  return res;
}
