#!/usr/bin/env python
# Software License Agreement (BSD License)
#
# Copyright (c) 2009, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of the Willow Garage nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import roslib
roslib.load_manifest('pr2_plugs_executive')

import rospy

import os
import sys
import time
import tf

import actionlib;
from pr2_common_action_msgs.msg import *
from pr2_plugs_msgs.msg import *
from pr2_controllers_msgs.msg import *
from actionlib_msgs.msg import *
from trajectory_msgs.msg import JointTrajectoryPoint
from pr2_plugs_actions.posestampedmath import PoseStampedMath
from math import pi


# Declare list of actions for easy construction
actions = [('vision_outlet_detection',VisionOutletDetectionAction),]

def main():
  rospy.init_node("plugs_executive")

  # construct tf listener
  transformer = tf.TransformListener(True, rospy.Duration(60.0))  
  preempt_timeout = rospy.Duration(5.0)

  # Construct action ac
  rospy.loginfo("Starting actions.")
  ac = dict()
  for (name,action) in actions:
    ac[name] = actionlib.SimpleActionClient(name,action)

  # Wait for all the ac to start
  for (name,action) in actions:
    print "Wait for server "+name
    ac[name].wait_for_server()


  detect_outlet_goal = VisionOutletDetectionGoal()

  rospy.loginfo("All actions started.")
  rospy.loginfo("Detecting outlet...")
  detect_outlet_goal.camera_name = "/forearm_camera_r"
  pose_base_outlet = PoseStampedMath()
  pose_base_outlet.fromEuler(-0.14, -0.82, 0.29, 0, 0, -pi/2)
  detect_outlet_goal.prior = pose_base_outlet.msg
  detect_outlet_goal.prior.header.stamp = rospy.Time.now()
  detect_outlet_goal.prior.header.frame_id = "base_link"
  #detect_outlet_goal.wall_normal = [0, 0, 0]
  if ac['vision_outlet_detection'].send_goal_and_wait(detect_outlet_goal, rospy.Duration(30.0), preempt_timeout) != GoalStatus.SUCCEEDED:
    rospy.logerr('Vision outlet detection failed')
    return


  outlet_pose = ac['vision_outlet_detection'].get_result().outlet_pose


  print "GOOD", outlet_pose

  return



if __name__ == "__main__":
  main()

